@@ -30,6 +30,7 @@
 #include "config-parser.h"
 #include "signals.h"
 #include "selinux.h"
+#include "dir-watch.h"
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-internals.h>
@@ -177,7 +178,7 @@ new_connection_callback (DBusServer     *server,
        * that we have to do this, people won't get it right
        * in general.
        */
-      dbus_connection_disconnect (new_connection);
+      dbus_connection_close (new_connection);
     }
 
   dbus_connection_set_max_received_size (new_connection,
@@ -478,6 +479,15 @@ process_config_every_time (BusContext      *context,
       goto failed;
     }
 
+  /* Drop existing conf-dir watches (if applicable) and watch all conf directories */
+
+  if (is_reload)
+    bus_drop_all_directory_watches ();
+
+  _dbus_list_foreach (bus_config_parser_get_conf_dirs (parser),
+		      (DBusForeachFunction) bus_watch_directory,
+		      NULL);
+
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   retval = TRUE;
 
@@ -720,8 +730,12 @@ bus_context_new (const DBusString *config_file,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
+
   if (parser != NULL)
-    bus_config_parser_unref (parser);
+    {
+      bus_config_parser_unref (parser);
+      parser = NULL;
+    }
   
   /* Here we change our credentials if required,
    * as soon as we've set up our sockets and pidfile
@@ -1103,7 +1117,7 @@ bus_context_check_security_policy (BusContext     *context,
                 (sender == NULL && !bus_connection_is_active (proposed_recipient)));
   _dbus_assert (type == DBUS_MESSAGE_TYPE_SIGNAL ||
                 addressed_recipient != NULL ||
-                strcmp (dbus_message_get_destination (message), DBUS_SERVICE_ORG_FREEDESKTOP_DBUS) == 0);
+                strcmp (dbus_message_get_destination (message), DBUS_SERVICE_DBUS) == 0);
   
   switch (type)
     {
@@ -1139,8 +1153,16 @@ bus_context_check_security_policy (BusContext     *context,
 				    dbus_message_get_interface (message),
 				    dbus_message_get_member (message),
 				    dbus_message_get_error_name (message),
-				    dest ? dest : DBUS_SERVICE_ORG_FREEDESKTOP_DBUS))
+				    dest ? dest : DBUS_SERVICE_DBUS, error))
         {
+
+	  if (dbus_error_is_set (error) &&
+	      dbus_error_has_name (error, DBUS_ERROR_NO_MEMORY))
+	    {
+	      return FALSE;
+	    }
+	  
+
           dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                           "An SELinux policy prevents this sender "
                           "from sending this message to this recipient "
@@ -1152,7 +1174,7 @@ bus_context_check_security_policy (BusContext     *context,
                           dbus_message_get_member (message) : "(unset)",
                           dbus_message_get_error_name (message) ?
                           dbus_message_get_error_name (message) : "(unset)",
-                          dest ? dest : DBUS_SERVICE_ORG_FREEDESKTOP_DBUS);
+                          dest ? dest : DBUS_SERVICE_DBUS);
           _dbus_verbose ("SELinux security check denying send to service\n");
           return FALSE;
         }
@@ -1192,7 +1214,7 @@ bus_context_check_security_policy (BusContext     *context,
            */
           if (proposed_recipient == NULL &&
               dbus_message_is_method_call (message,
-                                           DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
+                                           DBUS_INTERFACE_DBUS,
                                            "Hello"))
             {
               _dbus_verbose ("security check allowing %s message\n",
@@ -1277,7 +1299,7 @@ bus_context_check_security_policy (BusContext     *context,
                       dbus_message_get_member (message) : "(unset)",
                       dbus_message_get_error_name (message) ?
                       dbus_message_get_error_name (message) : "(unset)",
-                      dest ? dest : DBUS_SERVICE_ORG_FREEDESKTOP_DBUS);
+                      dest ? dest : DBUS_SERVICE_DBUS);
       _dbus_verbose ("security policy disallowing message due to sender policy\n");
       return FALSE;
     }
@@ -1304,7 +1326,7 @@ bus_context_check_security_policy (BusContext     *context,
                       dbus_message_get_member (message) : "(unset)",
                       dbus_message_get_error_name (message) ?
                       dbus_message_get_error_name (message) : "(unset)",
-                      dest ? dest : DBUS_SERVICE_ORG_FREEDESKTOP_DBUS,
+                      dest ? dest : DBUS_SERVICE_DBUS,
                       dbus_message_get_reply_serial (message),
                       requested_reply);
       _dbus_verbose ("security policy disallowing message due to recipient policy\n");
@@ -1323,7 +1345,7 @@ bus_context_check_security_policy (BusContext     *context,
                       "The destination service \"%s\" has a full message queue",
                       dest ? dest : (proposed_recipient ?
                                      bus_connection_get_name (proposed_recipient) : 
-                                     DBUS_SERVICE_ORG_FREEDESKTOP_DBUS));
+                                     DBUS_SERVICE_DBUS));
       _dbus_verbose ("security policy disallowing message due to full message queue\n");
       return FALSE;
     }
