@@ -199,12 +199,8 @@ bus_connection_disconnected (DBusConnection *connection)
       
       dbus_error_init (&error);
         
-      transaction = NULL;
-      while (transaction == NULL)
-        {
-          transaction = bus_transaction_new (d->connections->context);
-          _dbus_wait_for_memory ();
-        }
+      while ((transaction = bus_transaction_new (d->connections->context)) == NULL)
+        _dbus_wait_for_memory ();
         
       if (!bus_service_remove_owner (service, connection,
                                      transaction, &error))
@@ -501,7 +497,7 @@ bus_connections_unref (BusConnections *connections)
           connection = connections->incomplete->data;
 
           dbus_connection_ref (connection);
-          dbus_connection_disconnect (connection);
+          dbus_connection_close (connection);
           bus_connection_disconnected (connection);
           dbus_connection_unref (connection);
         }
@@ -516,7 +512,7 @@ bus_connections_unref (BusConnections *connections)
           connection = connections->completed->data;
 
           dbus_connection_ref (connection);
-          dbus_connection_disconnect (connection);
+          dbus_connection_close (connection);
           bus_connection_disconnected (connection);
           dbus_connection_unref (connection);
         }
@@ -653,7 +649,7 @@ bus_connections_setup_connection (BusConnections *connections,
        * completing authentication. But random may or may not really
        * help with that, a more elaborate solution might be required.
        */
-      dbus_connection_disconnect (connections->incomplete->data);
+      dbus_connection_close (connections->incomplete->data);
     }
   
   retval = TRUE;
@@ -739,7 +735,7 @@ bus_connections_expire_incomplete (BusConnections *connections)
           if (elapsed >= (double) auth_timeout)
             {
               _dbus_verbose ("Timing out authentication for connection %p\n", connection);
-              dbus_connection_disconnect (connection);
+              dbus_connection_close (connection);
             }
           else
             {
@@ -1091,7 +1087,7 @@ bus_connection_preallocate_oom_error (DBusConnection *connection)
   if (!dbus_message_set_error_name (message, DBUS_ERROR_NO_MEMORY) ||
       !dbus_message_set_destination (message, d->name) ||
       !dbus_message_set_sender (message,
-                                DBUS_SERVICE_ORG_FREEDESKTOP_DBUS))
+                                DBUS_SERVICE_DBUS))
     {
       dbus_connection_free_preallocated_send (connection, preallocated);
       dbus_message_unref (message);
@@ -1398,6 +1394,7 @@ bus_pending_reply_send_no_reply (BusConnections  *connections,
   DBusMessage *message;
   DBusMessageIter iter;
   dbus_bool_t retval;
+  const char *errmsg;
 
   retval = FALSE;
   
@@ -1414,9 +1411,10 @@ bus_pending_reply_send_no_reply (BusConnections  *connections,
   if (!dbus_message_set_error_name (message,
                                     DBUS_ERROR_NO_REPLY))
     goto out;
-  
-  dbus_message_append_iter_init (message, &iter);
-  if (!dbus_message_iter_append_string (&iter, "Message did not receive a reply (timeout by message bus)"))
+
+  errmsg = "Message did not receive a reply (timeout by message bus)";
+  dbus_message_iter_init_append (message, &iter);
+  if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &errmsg))
     goto out;
     
   if (!bus_transaction_send_from_driver (transaction, pending->will_get_reply,
@@ -1910,7 +1908,7 @@ bus_transaction_send_from_driver (BusTransaction *transaction,
                  dbus_message_get_error_name (message) ?
                  dbus_message_get_error_name (message) : "(no error name)");
                  
-  if (!dbus_message_set_sender (message, DBUS_SERVICE_ORG_FREEDESKTOP_DBUS))
+  if (!dbus_message_set_sender (message, DBUS_SERVICE_DBUS))
     return FALSE;
 
   if (bus_connection_is_active (connection))
