@@ -2,7 +2,7 @@
 /* driver.c  Bus client (driver)
  *
  * Copyright (C) 2003 CodeFactory AB
- * Copyright (C) 2003, 2004 Red Hat, Inc.
+ * Copyright (C) 2003, 2004, 2005 Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
  * 
@@ -27,10 +27,12 @@
 #include "driver.h"
 #include "dispatch.h"
 #include "services.h"
+#include "selinux.h"
 #include "signals.h"
 #include "utils.h"
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-internals.h>
+#include <dbus/dbus-marshal-recursive.h>
 #include <string.h>
 
 static dbus_bool_t bus_driver_send_welcome_message (DBusConnection *connection,
@@ -47,18 +49,19 @@ bus_driver_send_service_owner_changed (const char     *service_name,
 {
   DBusMessage *message;
   dbus_bool_t retval;
-  const char null_service[] = { '\000' };
+  const char *null_service;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
-  
-  _dbus_verbose ("sending service owner changed: %s [%s -> %s]\n",
+
+  null_service = "";
+  _dbus_verbose ("sending name owner changed: %s [%s -> %s]\n",
                  service_name, 
                  old_owner ? old_owner : null_service, 
                  new_owner ? new_owner : null_service);
 
-  message = dbus_message_new_signal (DBUS_PATH_ORG_FREEDESKTOP_DBUS,
-                                     DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                                     "ServiceOwnerChanged");
+  message = dbus_message_new_signal (DBUS_PATH_DBUS,
+                                     DBUS_INTERFACE_DBUS,
+                                     "NameOwnerChanged");
   
   if (message == NULL)
     {
@@ -66,13 +69,13 @@ bus_driver_send_service_owner_changed (const char     *service_name,
       return FALSE;
     }
   
-  if (!dbus_message_set_sender (message, DBUS_SERVICE_ORG_FREEDESKTOP_DBUS))
+  if (!dbus_message_set_sender (message, DBUS_SERVICE_DBUS))
     goto oom;
 
   if (!dbus_message_append_args (message,
-                                 DBUS_TYPE_STRING, service_name,
-                                 DBUS_TYPE_STRING, old_owner ? old_owner : null_service,
-                                 DBUS_TYPE_STRING, new_owner ? new_owner : null_service,
+                                 DBUS_TYPE_STRING, &service_name,
+                                 DBUS_TYPE_STRING, old_owner ? &old_owner : &null_service,
+                                 DBUS_TYPE_STRING, new_owner ? &new_owner : &null_service,
                                  DBUS_TYPE_INVALID))
     goto oom;
 
@@ -99,9 +102,9 @@ bus_driver_send_service_lost (DBusConnection *connection,
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
-  message = dbus_message_new_signal (DBUS_PATH_ORG_FREEDESKTOP_DBUS,
-                                     DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                                     "ServiceLost");
+  message = dbus_message_new_signal (DBUS_PATH_DBUS,
+                                     DBUS_INTERFACE_DBUS,
+                                     "NameLost");
   
   if (message == NULL)
     {
@@ -111,7 +114,7 @@ bus_driver_send_service_lost (DBusConnection *connection,
   
   if (!dbus_message_set_destination (message, bus_connection_get_name (connection)) ||
       !dbus_message_append_args (message,
-                                 DBUS_TYPE_STRING, service_name,
+                                 DBUS_TYPE_STRING, &service_name,
                                  DBUS_TYPE_INVALID))
     {
       dbus_message_unref (message);
@@ -142,9 +145,9 @@ bus_driver_send_service_acquired (DBusConnection *connection,
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
-  message = dbus_message_new_signal (DBUS_PATH_ORG_FREEDESKTOP_DBUS,
-                                     DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                                     "ServiceAcquired");
+  message = dbus_message_new_signal (DBUS_PATH_DBUS,
+                                     DBUS_INTERFACE_DBUS,
+                                     "NameAcquired");
 
   if (message == NULL)
     {
@@ -154,7 +157,7 @@ bus_driver_send_service_acquired (DBusConnection *connection,
   
   if (!dbus_message_set_destination (message, bus_connection_get_name (connection)) ||
       !dbus_message_append_args (message,
-                                 DBUS_TYPE_STRING, service_name,
+                                 DBUS_TYPE_STRING, &service_name,
                                  DBUS_TYPE_INVALID))
     {
       dbus_message_unref (message);
@@ -341,7 +344,7 @@ bus_driver_send_welcome_message (DBusConnection *connection,
     }
   
   if (!dbus_message_append_args (welcome,
-                                 DBUS_TYPE_STRING, name,
+                                 DBUS_TYPE_STRING, &name,
                                  DBUS_TYPE_INVALID))
     {
       dbus_message_unref (welcome);
@@ -349,7 +352,7 @@ bus_driver_send_welcome_message (DBusConnection *connection,
       return FALSE;
     }
 
-  _dbus_assert (dbus_message_has_signature (welcome, "s"));
+  _dbus_assert (dbus_message_has_signature (welcome, DBUS_TYPE_STRING_AS_STRING));
   
   if (!bus_transaction_send_from_driver (transaction, connection, welcome))
     {
@@ -374,6 +377,9 @@ bus_driver_handle_list_services (DBusConnection *connection,
   int len;
   char **services;
   BusRegistry *registry;
+  int i;
+  DBusMessageIter iter;
+  DBusMessageIter sub;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   
@@ -392,10 +398,12 @@ bus_driver_handle_list_services (DBusConnection *connection,
       BUS_SET_OOM (error);
       return FALSE;
     }
+
+  dbus_message_iter_init_append (reply, &iter);
   
-  if (!dbus_message_append_args (reply,
-                                 DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, services, len,
-                                 DBUS_TYPE_INVALID))
+  if (!dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY,
+                                         DBUS_TYPE_STRING_AS_STRING,
+                                         &sub))
     {
       dbus_free_string_array (services);
       dbus_message_unref (reply);
@@ -403,6 +411,41 @@ bus_driver_handle_list_services (DBusConnection *connection,
       return FALSE;
     }
 
+  {
+    /* Include the bus driver in the list */
+    const char *v_STRING = DBUS_SERVICE_DBUS;
+    if (!dbus_message_iter_append_basic (&sub, DBUS_TYPE_STRING,
+                                         &v_STRING))
+      {
+        dbus_free_string_array (services);
+        dbus_message_unref (reply);
+        BUS_SET_OOM (error);
+        return FALSE;
+      }
+  }
+  
+  i = 0;
+  while (i < len)
+    {
+      if (!dbus_message_iter_append_basic (&sub, DBUS_TYPE_STRING,
+                                           &services[i]))
+        {
+          dbus_free_string_array (services);
+          dbus_message_unref (reply);
+          BUS_SET_OOM (error);
+          return FALSE;
+        }
+      ++i;
+    }
+
+  if (!dbus_message_iter_close_container (&iter, &sub))
+    {
+      dbus_free_string_array (services);
+      dbus_message_unref (reply);
+      BUS_SET_OOM (error);
+      return FALSE;
+    }
+  
   dbus_free_string_array (services);
   
   if (!bus_transaction_send_from_driver (transaction, connection, reply))
@@ -426,7 +469,7 @@ bus_driver_handle_acquire_service (DBusConnection *connection,
 {
   DBusMessage *reply;
   DBusString service_name;
-  char *name;
+  const char *name;
   int service_reply;
   dbus_uint32_t flags;
   dbus_bool_t retval;
@@ -442,7 +485,7 @@ bus_driver_handle_acquire_service (DBusConnection *connection,
                               DBUS_TYPE_INVALID))
     return FALSE;
   
-  _dbus_verbose ("Trying to own service %s with flags 0x%x\n", name, flags);
+  _dbus_verbose ("Trying to own name %s with flags 0x%x\n", name, flags);
   
   retval = FALSE;
   reply = NULL;
@@ -462,7 +505,7 @@ bus_driver_handle_acquire_service (DBusConnection *connection,
       goto out;
     }
 
-  if (!dbus_message_append_args (reply, DBUS_TYPE_UINT32, service_reply, DBUS_TYPE_INVALID))
+  if (!dbus_message_append_args (reply, DBUS_TYPE_UINT32, &service_reply, DBUS_TYPE_INVALID))
     {
       BUS_SET_OOM (error);
       goto out;
@@ -477,7 +520,6 @@ bus_driver_handle_acquire_service (DBusConnection *connection,
   retval = TRUE;
   
  out:
-  dbus_free (name);
   if (reply)
     dbus_message_unref (reply);
   return retval;
@@ -493,7 +535,7 @@ bus_driver_handle_service_exists (DBusConnection *connection,
   DBusString service_name;
   BusService *service;
   dbus_bool_t service_exists;
-  char *name;
+  const char *name;
   dbus_bool_t retval;
   BusRegistry *registry;
 
@@ -508,7 +550,7 @@ bus_driver_handle_service_exists (DBusConnection *connection,
 
   retval = FALSE;
 
-  if (strcmp (name, DBUS_SERVICE_ORG_FREEDESKTOP_DBUS) == 0)
+  if (strcmp (name, DBUS_SERVICE_DBUS) == 0)
     {
       service_exists = TRUE;
     }
@@ -527,7 +569,7 @@ bus_driver_handle_service_exists (DBusConnection *connection,
     }
 
   if (!dbus_message_append_args (reply,
-                                 DBUS_TYPE_BOOLEAN, service_exists,
+                                 DBUS_TYPE_BOOLEAN, &service_exists,
                                  0))
     {
       BUS_SET_OOM (error);
@@ -545,7 +587,6 @@ bus_driver_handle_service_exists (DBusConnection *connection,
  out:
   if (reply)
     dbus_message_unref (reply);
-  dbus_free (name);
 
   return retval;
 }
@@ -557,7 +598,7 @@ bus_driver_handle_activate_service (DBusConnection *connection,
                                     DBusError      *error)
 {
   dbus_uint32_t flags;
-  char *name;
+  const char *name;
   dbus_bool_t retval;
   BusActivation *activation;
 
@@ -571,7 +612,7 @@ bus_driver_handle_activate_service (DBusConnection *connection,
                               DBUS_TYPE_INVALID))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
-      _dbus_verbose ("No memory to get arguments to ActivateService\n");
+      _dbus_verbose ("No memory to get arguments to StartServiceByName\n");
       return FALSE;
     }
 
@@ -588,7 +629,6 @@ bus_driver_handle_activate_service (DBusConnection *connection,
   retval = TRUE;
   
  out:
-  dbus_free (name);
   return retval;
 }
 
@@ -626,7 +666,7 @@ bus_driver_handle_add_match (DBusConnection *connection,
                              DBusError      *error)
 {
   BusMatchRule *rule;
-  char *text;
+  const char *text;
   DBusString str;
   BusMatchmaker *matchmaker;
   
@@ -677,7 +717,6 @@ bus_driver_handle_add_match (DBusConnection *connection,
     }
   
   bus_match_rule_unref (rule);
-  dbus_free (text);
   
   return TRUE;
 
@@ -685,8 +724,6 @@ bus_driver_handle_add_match (DBusConnection *connection,
   _DBUS_ASSERT_ERROR_IS_SET (error);
   if (rule)
     bus_match_rule_unref (rule);
-  if (text)
-    dbus_free (text);
   return FALSE;
 }
 
@@ -697,7 +734,7 @@ bus_driver_handle_remove_match (DBusConnection *connection,
                                 DBusError      *error)
 {
   BusMatchRule *rule;
-  char *text;
+  const char *text;
   DBusString str;
   BusMatchmaker *matchmaker;
   
@@ -733,7 +770,6 @@ bus_driver_handle_remove_match (DBusConnection *connection,
     goto failed;
 
   bus_match_rule_unref (rule);
-  dbus_free (text);
   
   return TRUE;
 
@@ -741,8 +777,6 @@ bus_driver_handle_remove_match (DBusConnection *connection,
   _DBUS_ASSERT_ERROR_IS_SET (error);
   if (rule)
     bus_match_rule_unref (rule);
-  if (text)
-    dbus_free (text);
   return FALSE;
 }
 
@@ -752,7 +786,7 @@ bus_driver_handle_get_service_owner (DBusConnection *connection,
 				     DBusMessage    *message,
 				     DBusError      *error)
 {
-  char *text;
+  const char *text;
   const char *base_name;
   DBusString str;
   BusRegistry *registry;
@@ -773,30 +807,41 @@ bus_driver_handle_get_service_owner (DBusConnection *connection,
 
   _dbus_string_init_const (&str, text);
   service = bus_registry_lookup (registry, &str);
-  if (service == NULL)
+  if (service == NULL &&
+      _dbus_string_equal_c_str (&str, DBUS_SERVICE_DBUS))
+    {
+      /* ORG_FREEDESKTOP_DBUS owns itself */
+      base_name = DBUS_SERVICE_DBUS;
+    }
+  else if (service == NULL)
     {
       dbus_set_error (error, 
-		      DBUS_ERROR_SERVICE_HAS_NO_OWNER,
-		      "Could not get owner of service '%s': no such service", text);
+                      DBUS_ERROR_NAME_HAS_NO_OWNER,
+                      "Could not get owner of name '%s': no such name", text);
       goto failed;
     }
+  else
+    {
+      base_name = bus_connection_get_name (bus_service_get_primary_owner (service));
+      if (base_name == NULL)
+        {
+          /* FIXME - how is this error possible? */
+          dbus_set_error (error,
+                          DBUS_ERROR_FAILED,
+                          "Could not determine unique name for '%s'", text);
+          goto failed;
+        }
+      _dbus_assert (*base_name == ':');      
+    }
 
-  base_name = bus_connection_get_name (bus_service_get_primary_owner (service));
-  if (base_name == NULL)
-    {
-      dbus_set_error (error,
-		      DBUS_ERROR_FAILED,
-		      "Could not determine base service for '%s'", text);
-      goto failed;
-    }
-  _dbus_assert (*base_name == ':');
+  _dbus_assert (base_name != NULL);
 
   reply = dbus_message_new_method_return (message);
   if (reply == NULL)
     goto oom;
 
   if (! dbus_message_append_args (reply, 
-				  DBUS_TYPE_STRING, base_name,
+				  DBUS_TYPE_STRING, &base_name,
 				  DBUS_TYPE_INVALID))
     goto oom;
   
@@ -804,7 +849,6 @@ bus_driver_handle_get_service_owner (DBusConnection *connection,
     goto oom;
 
   dbus_message_unref (reply);
-  dbus_free (text);
 
   return TRUE;
 
@@ -815,7 +859,6 @@ bus_driver_handle_get_service_owner (DBusConnection *connection,
   _DBUS_ASSERT_ERROR_IS_SET (error);
   if (reply)
     dbus_message_unref (reply);
-  dbus_free (text);
   return FALSE;
 }
 
@@ -825,13 +868,14 @@ bus_driver_handle_get_connection_unix_user (DBusConnection *connection,
                                             DBusMessage    *message,
                                             DBusError      *error)
 {
-  char *service;
+  const char *service;
   DBusString str;
   BusRegistry *registry;
   BusService *serv;
   DBusConnection *conn;
   DBusMessage *reply;
   unsigned long uid;
+  dbus_uint32_t uid32;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -852,8 +896,8 @@ bus_driver_handle_get_connection_unix_user (DBusConnection *connection,
   if (serv == NULL)
     {
       dbus_set_error (error, 
-		      DBUS_ERROR_SERVICE_HAS_NO_OWNER,
-		      "Could not get owner of service '%s': no such service", service);
+		      DBUS_ERROR_NAME_HAS_NO_OWNER,
+		      "Could not get UID of name '%s': no such name", service);
       goto failed;
     }
 
@@ -871,8 +915,9 @@ bus_driver_handle_get_connection_unix_user (DBusConnection *connection,
       goto failed;
     }
 
+  uid32 = uid;
   if (! dbus_message_append_args (reply,
-                                  DBUS_TYPE_UINT32, (dbus_uint32_t) uid,
+                                  DBUS_TYPE_UINT32, &uid32,
                                   DBUS_TYPE_INVALID))
     goto oom;
 
@@ -880,7 +925,6 @@ bus_driver_handle_get_connection_unix_user (DBusConnection *connection,
     goto oom;
 
   dbus_message_unref (reply);
-  dbus_free (service);
 
   return TRUE;
 
@@ -891,7 +935,6 @@ bus_driver_handle_get_connection_unix_user (DBusConnection *connection,
   _DBUS_ASSERT_ERROR_IS_SET (error);
   if (reply)
     dbus_message_unref (reply);
-  dbus_free (service);
   return FALSE;
 }
 
@@ -901,13 +944,14 @@ bus_driver_handle_get_connection_unix_process_id (DBusConnection *connection,
 						  DBusMessage    *message,
 						  DBusError      *error)
 {
-  char *service;
+  const char *service;
   DBusString str;
   BusRegistry *registry;
   BusService *serv;
   DBusConnection *conn;
   DBusMessage *reply;
   unsigned long pid;
+  dbus_uint32_t pid32;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -928,8 +972,8 @@ bus_driver_handle_get_connection_unix_process_id (DBusConnection *connection,
   if (serv == NULL)
     {
       dbus_set_error (error, 
-		      DBUS_ERROR_SERVICE_HAS_NO_OWNER,
-		      "Could not get owner of service '%s': no such service", service);
+		      DBUS_ERROR_NAME_HAS_NO_OWNER,
+		      "Could not get PID of name '%s': no such name", service);
       goto failed;
     }
 
@@ -947,8 +991,9 @@ bus_driver_handle_get_connection_unix_process_id (DBusConnection *connection,
       goto failed;
     }
 
+  pid32 = pid;
   if (! dbus_message_append_args (reply,
-                                  DBUS_TYPE_UINT32, (dbus_uint32_t) pid,
+                                  DBUS_TYPE_UINT32, &pid32,
                                   DBUS_TYPE_INVALID))
     goto oom;
 
@@ -956,7 +1001,6 @@ bus_driver_handle_get_connection_unix_process_id (DBusConnection *connection,
     goto oom;
 
   dbus_message_unref (reply);
-  dbus_free (service);
 
   return TRUE;
 
@@ -967,7 +1011,79 @@ bus_driver_handle_get_connection_unix_process_id (DBusConnection *connection,
   _DBUS_ASSERT_ERROR_IS_SET (error);
   if (reply)
     dbus_message_unref (reply);
-  dbus_free (service);
+  return FALSE;
+}
+
+static dbus_bool_t
+bus_driver_handle_get_connection_selinux_security_context (DBusConnection *connection,
+							   BusTransaction *transaction,
+							   DBusMessage    *message,
+							   DBusError      *error)
+{
+  const char *service;
+  DBusString str;
+  BusRegistry *registry;
+  BusService *serv;
+  DBusConnection *conn;
+  DBusMessage *reply;
+  BusSELinuxID *context;
+
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+  registry = bus_connection_get_registry (connection);
+
+  service = NULL;
+  reply = NULL;
+
+  if (! dbus_message_get_args (message, error,
+			       DBUS_TYPE_STRING, &service,
+			       DBUS_TYPE_INVALID))
+      goto failed;
+
+  _dbus_verbose ("asked for security context of connection %s\n", service);
+
+  _dbus_string_init_const (&str, service);
+  serv = bus_registry_lookup (registry, &str);
+  if (serv == NULL)
+    {
+      dbus_set_error (error, 
+		      DBUS_ERROR_NAME_HAS_NO_OWNER,
+		      "Could not get security context of name '%s': no such name", service);
+      goto failed;
+    }
+
+  conn = bus_service_get_primary_owner (serv);
+
+  reply = dbus_message_new_method_return (message);
+  if (reply == NULL)
+    goto oom;
+
+  context = bus_connection_get_selinux_id (conn);
+  if (!context)
+    {
+      dbus_set_error (error,
+                      DBUS_ERROR_SELINUX_SECURITY_CONTEXT_UNKNOWN,
+                      "Could not determine security context for '%s'", service);
+      goto failed;
+    }
+
+  if (! bus_selinux_append_context (reply, context, error))
+    goto failed;
+
+  if (! bus_transaction_send_from_driver (transaction, connection, reply))
+    goto oom;
+
+  dbus_message_unref (reply);
+
+  return TRUE;
+
+ oom:
+  BUS_SET_OOM (error);
+
+ failed:
+  _DBUS_ASSERT_ERROR_IS_SET (error);
+  if (reply)
+    dbus_message_unref (reply);
   return FALSE;
 }
 
@@ -1004,31 +1120,209 @@ bus_driver_handle_reload_config (DBusConnection *connection,
 struct
 {
   const char *name;
+  const char *in_args;
+  const char *out_args;
   dbus_bool_t (* handler) (DBusConnection *connection,
                            BusTransaction *transaction,
                            DBusMessage    *message,
                            DBusError      *error);
 } message_handlers[] = {
-  { "AcquireService", bus_driver_handle_acquire_service },
-  { "ActivateService", bus_driver_handle_activate_service },
-  { "Hello", bus_driver_handle_hello },
-  { "ServiceExists", bus_driver_handle_service_exists },
-  { "ListServices", bus_driver_handle_list_services },
-  { "AddMatch", bus_driver_handle_add_match },
-  { "RemoveMatch", bus_driver_handle_remove_match },
-  { "GetServiceOwner", bus_driver_handle_get_service_owner },
-  { "GetConnectionUnixUser", bus_driver_handle_get_connection_unix_user },
-  { "GetConnectionUnixProcessID", bus_driver_handle_get_connection_unix_process_id },
-  { "ReloadConfig", bus_driver_handle_reload_config }
+  { "RequestName",
+    DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_UINT32_AS_STRING,
+    DBUS_TYPE_UINT32_AS_STRING,
+    bus_driver_handle_acquire_service },
+  { "StartServiceByName",
+    DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_UINT32_AS_STRING,
+    DBUS_TYPE_UINT32_AS_STRING,
+    bus_driver_handle_activate_service },
+  { "Hello",
+    "",
+    DBUS_TYPE_STRING_AS_STRING,
+    bus_driver_handle_hello },
+  { "NameHasOwner",
+    DBUS_TYPE_STRING_AS_STRING,
+    DBUS_TYPE_BOOLEAN_AS_STRING,
+    bus_driver_handle_service_exists },
+  { "ListNames",
+    "",
+    DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
+    bus_driver_handle_list_services },
+  { "AddMatch",
+    DBUS_TYPE_STRING_AS_STRING,
+    "",
+    bus_driver_handle_add_match },
+  { "RemoveMatch",
+    DBUS_TYPE_STRING_AS_STRING,
+    "",
+    bus_driver_handle_remove_match },
+  { "GetNameOwner",
+    DBUS_TYPE_STRING_AS_STRING,
+    DBUS_TYPE_STRING_AS_STRING,
+    bus_driver_handle_get_service_owner },
+  { "GetConnectionUnixUser",
+    DBUS_TYPE_STRING_AS_STRING,
+    DBUS_TYPE_UINT32_AS_STRING,
+    bus_driver_handle_get_connection_unix_user },
+  { "GetConnectionUnixProcessID",
+    DBUS_TYPE_STRING_AS_STRING,
+    DBUS_TYPE_UINT32_AS_STRING,
+    bus_driver_handle_get_connection_unix_process_id },
+  { "GetConnectionSELinuxSecurityContext",
+    DBUS_TYPE_STRING_AS_STRING,
+    DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_BYTE_AS_STRING,
+    bus_driver_handle_get_connection_selinux_security_context },
+  { "ReloadConfig",
+    "",
+    "",
+    bus_driver_handle_reload_config }
 };
 
+static dbus_bool_t
+write_args_for_direction (DBusString *xml,
+			  const char *signature,
+			  dbus_bool_t in)
+{
+  DBusTypeReader typereader;
+  DBusString sigstr;
+  int current_type;
+  
+  _dbus_string_init_const (&sigstr, signature);
+  _dbus_type_reader_init_types_only (&typereader, &sigstr, 0);
+      
+  while ((current_type = _dbus_type_reader_get_current_type (&typereader)) != DBUS_TYPE_INVALID)
+    {
+      const DBusString *subsig;
+      int start, len;
+
+      _dbus_type_reader_get_signature (&typereader, &subsig, &start, &len);
+      if (!_dbus_string_append_printf (xml, "      <arg direction=\"%s\" type=\"",
+				       in ? "in" : "out"))
+	goto oom;
+      if (!_dbus_string_append_len (xml,
+				    _dbus_string_get_const_data (subsig) + start,
+				    len))
+	goto oom;
+      if (!_dbus_string_append (xml, "\"/>\n"))
+	goto oom;
+
+      _dbus_type_reader_next (&typereader);
+    }
+  return TRUE;
+ oom:
+  return FALSE;
+}
+
+static dbus_bool_t
+bus_driver_handle_introspect (DBusConnection *connection,
+                              BusTransaction *transaction,
+                              DBusMessage    *message,
+                              DBusError      *error)
+{
+  DBusString xml;
+  DBusMessage *reply;
+  const char *v_STRING;
+  int i;
+
+  _dbus_verbose ("Introspect() on bus driver\n");
+  
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+
+  reply = NULL;
+
+  if (! dbus_message_get_args (message, error,
+			       DBUS_TYPE_INVALID))
+    {
+      _DBUS_ASSERT_ERROR_IS_SET (error);
+      return FALSE;
+    }
+
+  if (!_dbus_string_init (&xml))
+    {
+      BUS_SET_OOM (error);
+      return FALSE;
+    }
+
+  if (!_dbus_string_append (&xml, DBUS_INTROSPECT_1_0_XML_DOCTYPE_DECL_NODE))
+    goto oom;
+  if (!_dbus_string_append (&xml, "<node>\n"))
+    goto oom;
+  if (!_dbus_string_append_printf (&xml, "  <interface name=\"%s\">\n", DBUS_INTERFACE_INTROSPECTABLE))
+    goto oom;
+  if (!_dbus_string_append (&xml, "    <method name=\"Introspect\">\n"))
+    goto oom;
+  if (!_dbus_string_append_printf (&xml, "      <arg name=\"data\" direction=\"out\" type=\"%s\"/>\n", DBUS_TYPE_STRING_AS_STRING))
+    goto oom;
+  if (!_dbus_string_append (&xml, "    </method>\n"))
+    goto oom;
+  if (!_dbus_string_append (&xml, "  </interface>\n"))
+    goto oom;
+
+  if (!_dbus_string_append_printf (&xml, "  <interface name=\"%s\">\n",
+                                   DBUS_INTERFACE_DBUS))
+    goto oom;
+
+  i = 0;
+  while (i < _DBUS_N_ELEMENTS (message_handlers))
+    {
+	  
+      if (!_dbus_string_append_printf (&xml, "    <method name=\"%s\">\n",
+                                       message_handlers[i].name))
+        goto oom;
+
+      if (!write_args_for_direction (&xml, message_handlers[i].in_args, TRUE))
+	goto oom;
+
+      if (!write_args_for_direction (&xml, message_handlers[i].out_args, FALSE))
+	goto oom;
+
+      if (!_dbus_string_append (&xml, "    </method>\n"))
+	goto oom;
+      
+      ++i;
+    }
+  
+  if (!_dbus_string_append (&xml, "  </interface>\n"))
+    goto oom;
+  
+  if (!_dbus_string_append (&xml, "</node>\n"))
+    goto oom;
+
+  reply = dbus_message_new_method_return (message);
+  if (reply == NULL)
+    goto oom;
+
+  v_STRING = _dbus_string_get_const_data (&xml);
+  if (! dbus_message_append_args (reply,
+                                  DBUS_TYPE_STRING, &v_STRING,
+                                  DBUS_TYPE_INVALID))
+    goto oom;
+
+  if (! bus_transaction_send_from_driver (transaction, connection, reply))
+    goto oom;
+
+  dbus_message_unref (reply);
+  _dbus_string_free (&xml);
+
+  return TRUE;
+
+ oom:
+  BUS_SET_OOM (error);
+
+  if (reply)
+    dbus_message_unref (reply);
+
+  _dbus_string_free (&xml);
+  
+  return FALSE;
+}
+
 dbus_bool_t
 bus_driver_handle_message (DBusConnection *connection,
                            BusTransaction *transaction,
 			   DBusMessage    *message,
                            DBusError      *error)
 {
-  const char *name, *sender;
+  const char *name, *sender, *interface;
   int i;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
@@ -1039,17 +1333,25 @@ bus_driver_handle_message (DBusConnection *connection,
       return TRUE; /* we just ignore this */
     }
 
-  _dbus_assert (dbus_message_get_interface (message) != NULL);
+  if (dbus_message_is_method_call (message,
+                                   DBUS_INTERFACE_INTROSPECTABLE,
+                                   "Introspect"))
+    return bus_driver_handle_introspect (connection, transaction, message, error);
+  
+  interface = dbus_message_get_interface (message);
+  if (interface == NULL)
+    interface = DBUS_INTERFACE_DBUS;
+  
   _dbus_assert (dbus_message_get_member (message) != NULL);
-
+  
   name = dbus_message_get_member (message);
   sender = dbus_message_get_sender (message);
   
-  if (strcmp (dbus_message_get_interface (message),
-              DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS) != 0)
+  if (strcmp (interface,
+              DBUS_INTERFACE_DBUS) != 0)
     {
       _dbus_verbose ("Driver got message to unknown interface \"%s\"\n",
-                     dbus_message_get_interface (message));
+                     interface);
       goto unknown;
     }
   
@@ -1058,19 +1360,29 @@ bus_driver_handle_message (DBusConnection *connection,
   
   /* security checks should have kept this from getting here */
   _dbus_assert (sender != NULL || strcmp (name, "Hello") == 0);
-
-  if (dbus_message_get_reply_serial (message) != 0)
-    {
-      _dbus_verbose ("Client sent a reply to the bus driver, ignoring it\n");
-      return TRUE;
-    }
   
   i = 0;
   while (i < _DBUS_N_ELEMENTS (message_handlers))
     {
       if (strcmp (message_handlers[i].name, name) == 0)
         {
-          _dbus_verbose ("Running driver handler for %s\n", name);
+          _dbus_verbose ("Found driver handler for %s\n", name);
+
+          if (!dbus_message_has_signature (message, message_handlers[i].in_args))
+            {
+              _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+              _dbus_verbose ("Call to %s has wrong args (%s, expected %s)\n",
+                             name, dbus_message_get_signature (message),
+                             message_handlers[i].in_args);
+              
+              dbus_set_error (error, DBUS_ERROR_INVALID_ARGS,
+                              "Call to %s has wrong args (%s, expected %s)\n",
+                              name, dbus_message_get_signature (message),
+                              message_handlers[i].in_args);
+              _DBUS_ASSERT_ERROR_IS_SET (error);
+              return FALSE;
+            }
+          
           if ((* message_handlers[i].handler) (connection, transaction, message, error))
             {
               _DBUS_ASSERT_ERROR_IS_CLEAR (error);
@@ -1094,7 +1406,7 @@ bus_driver_handle_message (DBusConnection *connection,
 
   dbus_set_error (error, DBUS_ERROR_UNKNOWN_METHOD,
                   "%s does not understand message %s",
-                  DBUS_SERVICE_ORG_FREEDESKTOP_DBUS, name);
+                  DBUS_SERVICE_DBUS, name);
   
   return FALSE;
 }
