@@ -24,6 +24,7 @@
 #include <dbus/dbus-hash.h>
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-mempool.h>
+#include <dbus/dbus-marshal-validate.h>
 
 #include "driver.h"
 #include "services.h"
@@ -156,13 +157,18 @@ bus_registry_ensure (BusRegistry               *registry,
 
   service->registry = registry;  
   service->refcount = 1;
-  
+
+  _dbus_verbose ("copying string %p '%s' to service->name\n",
+                 service_name, _dbus_string_get_const_data (service_name));
   if (!_dbus_string_copy_data (service_name, &service->name))
     {
       _dbus_mem_pool_dealloc (registry->service_pool, service);
       BUS_SET_OOM (error);
       return NULL;
     }
+  _dbus_verbose ("copied string %p '%s' to '%s'\n",
+                 service_name, _dbus_string_get_const_data (service_name),
+                 service->name);
 
   if (!bus_driver_send_service_owner_changed (service->name, 
 					      NULL,
@@ -278,12 +284,14 @@ bus_registry_acquire_service (BusRegistry      *registry,
   
   retval = FALSE;
 
-  if (_dbus_string_get_length (service_name) == 0)
+  if (!_dbus_validate_bus_name (service_name, 0,
+                                _dbus_string_get_length (service_name)))
     {
-      dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
-                      "Zero-length service name is not allowed");
+      dbus_set_error (error, DBUS_ERROR_INVALID_ARGS,
+                      "Requested bus name \"%s\" is not valid",
+                      _dbus_string_get_const_data (service_name));
       
-      _dbus_verbose ("Attempt to acquire zero-length service name\n");
+      _dbus_verbose ("Attempt to acquire invalid service name\n");
       
       goto out;
     }
@@ -291,7 +299,7 @@ bus_registry_acquire_service (BusRegistry      *registry,
   if (_dbus_string_get_byte (service_name, 0) == ':')
     {
       /* Not allowed; only base services can start with ':' */
-      dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
+      dbus_set_error (error, DBUS_ERROR_INVALID_ARGS,
                       "Cannot acquire a service starting with ':' such as \"%s\"",
                       _dbus_string_get_const_data (service_name));
       
@@ -311,8 +319,15 @@ bus_registry_acquire_service (BusRegistry      *registry,
                                      service_name);
 
   if (!bus_selinux_allows_acquire_service (connection, sid,
-					   _dbus_string_get_const_data (service_name)))
+					   _dbus_string_get_const_data (service_name), error))
     {
+
+      if (dbus_error_is_set (error) &&
+	  dbus_error_has_name (error, DBUS_ERROR_NO_MEMORY))
+	{
+	  goto out;
+	}
+
       dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                       "Connection \"%s\" is not allowed to own the service \"%s\" due "
                       "to SELinux policy",
@@ -370,14 +385,14 @@ bus_registry_acquire_service (BusRegistry      *registry,
       _dbus_assert (current_owner == connection);
 
       bus_service_set_prohibit_replacement (service,
-					    (flags & DBUS_SERVICE_FLAG_PROHIBIT_REPLACEMENT));      
+					    (flags & DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT));      
 			
-      *result = DBUS_SERVICE_REPLY_PRIMARY_OWNER;      
+      *result = DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER;      
     }
   else if (old_owner == connection)
-    *result = DBUS_SERVICE_REPLY_ALREADY_OWNER;
-  else if (!((flags & DBUS_SERVICE_FLAG_REPLACE_EXISTING)))
-    *result = DBUS_SERVICE_REPLY_SERVICE_EXISTS;
+    *result = DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER;
+  else if (!((flags & DBUS_NAME_FLAG_REPLACE_EXISTING)))
+    *result = DBUS_REQUEST_NAME_REPLY_EXISTS;
   else if (bus_service_get_prohibit_replacement (service))
     {
       /* Queue the connection */
@@ -385,14 +400,14 @@ bus_registry_acquire_service (BusRegistry      *registry,
                                   transaction, error))
         goto out;
       
-      *result = DBUS_SERVICE_REPLY_IN_QUEUE;
+      *result = DBUS_REQUEST_NAME_REPLY_IN_QUEUE;
     }
   else
     {
       /* Replace the current owner */
 
       /* We enqueue the new owner and remove the first one because
-       * that will cause ServiceAcquired and ServiceLost messages to
+       * that will cause NameAcquired and NameLost messages to
        * be sent.
        */
       
@@ -405,7 +420,7 @@ bus_registry_acquire_service (BusRegistry      *registry,
         goto out;
       
       _dbus_assert (connection == bus_service_get_primary_owner (service));
-      *result = DBUS_SERVICE_REPLY_PRIMARY_OWNER;
+      *result = DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER;
     }
 
   activation = bus_context_get_activation (registry->context);
