@@ -32,15 +32,12 @@ G_BEGIN_DECLS
 void dbus_set_g_error (GError   **gerror,
                        DBusError *derror);
 
-#define DBUS_TYPE_CONNECTION (dbus_connection_get_g_type ())
-#define DBUS_TYPE_MESSAGE    (dbus_message_get_g_type ())
-GType dbus_connection_get_g_type (void) G_GNUC_CONST;
-GType dbus_message_get_g_type    (void) G_GNUC_CONST;
-
-#define DBUS_TYPE_G_CONNECTION (dbus_g_connection_get_g_type ())
-#define DBUS_TYPE_G_MESSAGE    (dbus_g_message_get_g_type ())
-GType dbus_g_connection_get_g_type (void) G_GNUC_CONST;
-GType dbus_g_message_get_g_type    (void) G_GNUC_CONST;
+#define DBUS_TYPE_CONNECTION      (dbus_connection_get_g_type ())
+#define DBUS_TYPE_MESSAGE         (dbus_message_get_g_type ())
+#define DBUS_TYPE_PENDING_CALL    (dbus_pending_call_get_g_type ())
+GType dbus_connection_get_g_type   (void) G_GNUC_CONST;
+GType dbus_message_get_g_type      (void) G_GNUC_CONST;
+GType dbus_pending_call_get_g_type (void) G_GNUC_CONST;
 
 void            dbus_connection_setup_with_g_main (DBusConnection  *connection,
                                                    GMainContext    *context);
