@@ -25,18 +25,29 @@
 #include "dbus-gidl.h"
 #include "dbus-gparser.h"
 #include "dbus-gutils.h"
+#include "dbus-glib-tool.h"
+#include "dbus-binding-tool-glib.h"
 #include <locale.h>
 #include <libintl.h>
 #define _(x) dgettext (GETTEXT_PACKAGE, x)
 #define N_(x) x
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
+#include <sys/stat.h>
 #include <string.h>
 
 #ifdef DBUS_BUILD_TESTS
 static void run_all_tests (const char *test_data_dir);
 #endif
 
+typedef enum {
+  DBUS_BINDING_OUTPUT_NONE,
+  DBUS_BINDING_OUTPUT_PRETTY,
+  DBUS_BINDING_OUTPUT_GLIB_SERVER,
+  DBUS_BINDING_OUTPUT_GLIB_CLIENT,
+} DBusBindingOutputMode;
+
 static void
 indent (int depth)
 {
@@ -57,7 +68,7 @@ pretty_print_list (GSList *list,
                    int     depth)
 {
   GSList *tmp;
-
+  
   tmp = list;
   while (tmp != NULL)
     {
@@ -99,13 +110,26 @@ pretty_print (BaseInfo *base,
     case INFO_TYPE_INTERFACE:
       {
         InterfaceInfo *i = (InterfaceInfo*) base;
+	GSList *annotations, *elt;
 
         g_assert (name != NULL);
 
         printf (_("interface \"%s\" {\n"), name);
 
+	annotations = interface_info_get_annotations (i);
+	for (elt = annotations; elt; elt = elt->next)
+	  {
+	    const char *name = elt->data;
+	    const char *value = interface_info_get_annotation (i, name);
+
+	    printf (_(" (binding \"%s\": \"%s\") "),
+		    name, value);
+	  }
+	g_slist_free (annotations);
+
         pretty_print_list (interface_info_get_methods (i), depth + 1);
         pretty_print_list (interface_info_get_signals (i), depth + 1);
+        pretty_print_list (interface_info_get_properties (i), depth + 1);
 
         indent (depth);
         printf ("}\n");
@@ -114,10 +138,21 @@ pretty_print (BaseInfo *base,
     case INFO_TYPE_METHOD:
       {
         MethodInfo *m = (MethodInfo*) base;
+	GSList *annotations, *elt;
 
         g_assert (name != NULL);
 
-        printf (_("method \"%s\" (\n"), name);
+	annotations = method_info_get_annotations (m);
+        printf (_("method \"%s\""), name);
+	for (elt = annotations; elt; elt = elt->next)
+	  {
+	    const char *name = elt->data;
+	    const char *value = method_info_get_annotation (m, name);
+
+	    printf (_(" (annotation \"%s\": \"%s\") "),
+		    name, value);
+	  }
+	g_slist_free (annotations);
 
         pretty_print_list (method_info_get_args (m), depth + 1);
 
@@ -139,15 +174,31 @@ pretty_print (BaseInfo *base,
         printf (")\n");
       }
       break;
+    case INFO_TYPE_PROPERTY:
+      {
+        PropertyInfo *a = (PropertyInfo*) base;
+        const char *pt = property_info_get_type (a);
+        PropertyAccessFlags acc = property_info_get_access (a);
+
+        printf ("%s%s %s",
+                acc & PROPERTY_READ ? "read" : "",
+                acc & PROPERTY_WRITE ? "write" : "",
+                pt);
+        if (name)
+          printf (" %s\n", name);
+        else
+          printf ("\n");
+      }
+      break;
     case INFO_TYPE_ARG:
       {
         ArgInfo *a = (ArgInfo*) base;
-        int at = arg_info_get_type (a);
+        const char *at = arg_info_get_type (a);
         ArgDirection d = arg_info_get_direction (a);
 
         printf ("%s %s",
                 d == ARG_IN ? "in" : "out",
-                _dbus_gutils_type_to_string (at));
+                at);
         if (name)
           printf (" %s\n", name);
         else
@@ -157,18 +208,52 @@ pretty_print (BaseInfo *base,
     }
 }
 
+GQuark
+dbus_binding_tool_error_quark (void)
+{
+  static GQuark quark = 0;
+  if (!quark)
+    quark = g_quark_from_static_string ("dbus_binding_tool_error");
+
+  return quark;
+}
+
+static void lose (const char *fmt, ...) G_GNUC_NORETURN G_GNUC_PRINTF (1, 2);
+static void lose_gerror (const char *prefix, GError *error) G_GNUC_NORETURN;
+
+static void
+lose (const char *str, ...)
+{
+  va_list args;
+
+  va_start (args, str);
+
+  vfprintf (stderr, str, args);
+  fputc ('\n', stderr);
+
+  va_end (args);
+
+  exit (1);
+}
+
+static void
+lose_gerror (const char *prefix, GError *error) 
+{
+  lose ("%s: %s", prefix, error->message);
+}
+
 static void
 usage (int ecode)
 {
-  fprintf (stderr, "dbus-glib-tool [--version] [--help]\n");
+  fprintf (stderr, "dbus-binding-tool [--version] [--help] [--pretty-print]\n");
   exit (ecode);
 }
 
 static void
 version (void)
 {
-  printf ("D-BUS GLib Tool %s\n"
-          "Copyright (C) 2003, 2004 Red Hat, Inc.\n"
+  printf ("D-BUS Binding Tool %s\n"
+          "Copyright (C) 2003-2005 Red Hat, Inc.\n"
           "This is free software; see the source for copying conditions.\n"
           "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
           VERSION);
@@ -179,21 +264,37 @@ int
 main (int argc, char **argv)
 {
   const char *prev_arg;
+  const char *output_file;
+  const char *prefix;
+  char *output_file_tmp;
   int i;
   GSList *files;
+  DBusBindingOutputMode outputmode;
   gboolean end_of_args;
   GSList *tmp;
-  gboolean just_pretty_print;
+  GIOChannel *channel;
+  GError *error;
+  time_t newest_src;
+  struct stat srcbuf;
+  struct stat targetbuf;
+  gboolean force;
+  gboolean ignore_unsupported;
 
   setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, DBUS_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE); 
 
-  just_pretty_print = FALSE;
+  g_type_init ();
+
+  outputmode = DBUS_BINDING_OUTPUT_NONE;
   end_of_args = FALSE;
   files = NULL;
   prev_arg = NULL;
+  output_file = NULL;
+  prefix = "";
+  ignore_unsupported = FALSE;
+  force = FALSE;
   i = 1;
   while (i < argc)
     {
@@ -207,12 +308,34 @@ main (int argc, char **argv)
             usage (0);
           else if (strcmp (arg, "--version") == 0)
             version ();
+          else if (strcmp (arg, "--force") == 0)
+            force = TRUE;
 #ifdef DBUS_BUILD_TESTS
           else if (strcmp (arg, "--self-test") == 0)
             run_all_tests (NULL);
 #endif /* DBUS_BUILD_TESTS */
-          else if (strcmp (arg, "--pretty-print") == 0)
-            just_pretty_print = TRUE;
+          else if (strncmp (arg, "--mode=", 7) == 0)
+            {
+	      const char *mode = arg + 7;
+	      if (!strcmp (mode, "pretty"))
+		outputmode = DBUS_BINDING_OUTPUT_PRETTY;
+	      else if (!strcmp (mode, "glib-server"))
+		outputmode = DBUS_BINDING_OUTPUT_GLIB_SERVER;
+	      else if (!strcmp (mode, "glib-client"))
+		outputmode = DBUS_BINDING_OUTPUT_GLIB_CLIENT;
+	      else
+		usage (1);
+	    }
+          else if (strcmp (arg, "--ignore-unsupported") == 0)
+            ignore_unsupported = TRUE;
+	  else if (strncmp (arg, "--output=", 9) == 0)
+	    {
+	      output_file = arg + 9;
+	    }
+          else if (strncmp (arg, "--prefix=", 9) == 0)
+            {
+              prefix = arg + 9;
+            }
           else if (arg[0] == '-' &&
                    arg[1] == '-' &&
                    arg[2] == '\0')
@@ -234,10 +357,47 @@ main (int argc, char **argv)
       ++i;
     }
 
+  error = NULL;
+
   files = g_slist_reverse (files);
 
-  tmp = files;
-  while (tmp != NULL)
+  if (output_file && !force)
+    {
+      newest_src = 0;
+      for (tmp = files; tmp != NULL; tmp = tmp->next)
+	{
+	  const char *filename;
+
+	  filename = tmp->data;
+	  if (stat (filename, &srcbuf) < 0)
+	    lose ("Couldn't stat %s: %s", filename, g_strerror (errno));
+
+	  if (srcbuf.st_mtime > newest_src)
+	    newest_src = srcbuf.st_mtime;
+	}
+
+      if (stat (output_file, &targetbuf) > 0
+	  && targetbuf.st_mtime >= newest_src)
+	exit (0);
+    }
+  
+  if (output_file)
+    {
+      output_file_tmp = g_strconcat (output_file, ".tmp", NULL);
+
+      if (!(channel = g_io_channel_new_file (output_file_tmp, "w", &error)))
+	lose_gerror (_("Couldn't open temporary file"), error);
+    }
+  else
+    {
+      channel = g_io_channel_unix_new (fileno (stdout));
+      output_file_tmp = NULL; /* silence gcc */
+    }
+  if (!g_io_channel_set_encoding (channel, NULL, &error))
+    lose_gerror (_("Couldn't set channel encoding to NULL"), error);
+
+
+  for (tmp = files; tmp != NULL; tmp = tmp->next)
     {
       NodeInfo *node;
       GError *error;
@@ -250,34 +410,44 @@ main (int argc, char **argv)
                                          &error);
       if (node == NULL)
         {
-          g_assert (error != NULL);
-          fprintf (stderr, _("Unable to load \"%s\": %s\n"),
-                   filename, error->message);
-          g_error_free (error);
-          exit (1);
-        }
-      else if (just_pretty_print)
-        {
-          pretty_print ((BaseInfo*) node, 0);
+	  lose_gerror (_("Unable to load \"%s\""), error);
         }
       else
-        {
-          /* FIXME process the file to generate metadata variable
-           * definition rather than just printing it.
-           * i.e. we want to create DBusGObjectInfo.
-           * This probably requires extending the introspection XML format to
-           * allow a "native function name":
-           *  <method name="Frobate" native="my_object_frobate">
-           */
-          pretty_print ((BaseInfo*) node, 0);
-        }
+	{
+	  switch (outputmode)
+	    {
+	    case DBUS_BINDING_OUTPUT_PRETTY:
+	      pretty_print ((BaseInfo*) node, 0);
+	      break;
+	    case DBUS_BINDING_OUTPUT_GLIB_SERVER:
+	      if (!dbus_binding_tool_output_glib_server ((BaseInfo *) node, channel, prefix, &error))
+		lose_gerror (_("Compilation failed"), error);
+	      break;
+	    case DBUS_BINDING_OUTPUT_GLIB_CLIENT:
+	      if (!dbus_binding_tool_output_glib_client ((BaseInfo *) node, channel, ignore_unsupported, &error))
+		lose_gerror (_("Compilation failed"), error);
+	      break;
+	    case DBUS_BINDING_OUTPUT_NONE:
+	      break;
+	    }
+	}
 
       if (node)
         node_info_unref (node);
-      
-      tmp = tmp->next;
     }
-  
+
+  if (g_io_channel_shutdown (channel, TRUE, &error) != G_IO_STATUS_NORMAL)
+    lose_gerror (_("Failed to shutdown IO channel"), error);
+  g_io_channel_unref (channel);
+
+  if (output_file)
+    {
+      if (rename (output_file_tmp, output_file) < 0)
+	lose ("Failed to rename %s to %s: %s", output_file_tmp, output_file,
+	      g_strerror (errno));
+      g_free (output_file_tmp);
+    }
+
   return 0;
 }
 
@@ -286,8 +456,7 @@ main (int argc, char **argv)
 static void
 test_die (const char *failure)
 {
-  fprintf (stderr, "Unit test failed: %s\n", failure);
-  exit (1);
+  lose ("Unit test failed: %s", failure);
 }
 
 /**
@@ -313,11 +482,11 @@ run_all_tests (const char *test_data_dir)
   else
     printf ("No test data!\n");
 
-  printf ("%s: running gtool tests\n", "dbus-glib-tool");
+  printf ("%s: running binding tests\n", "dbus-binding-tool");
   if (!_dbus_gtool_test (test_data_dir))
     test_die ("gtool");
 
-  printf ("%s: completed successfully\n", "dbus-glib-tool");
+  printf ("%s: completed successfully\n", "dbus-binding-tool");
 }
 
 #endif /* DBUS_BUILD_TESTS */
