@@ -23,10 +23,7 @@
 #include <string.h>
 #include <config.h>
 #include "dbus-tree-view.h"
-
-#include <libintl.h>
-#define _(x) dgettext (GETTEXT_PACKAGE, x)
-#define N_(x) x
+#include <glib/gi18n.h>
 
 enum
 {
@@ -52,12 +49,7 @@ model_new (void)
   GtkTreeStore *store;
 
   store = gtk_tree_store_new (MODEL_COLUMN_LAST,
-                              G_TYPE_POINTER);
-  /* FIXME, BASE_INFO_TYPE doesn't work right (crashes),
-   * G_TYPE_POINTER has a memleak. BASE_INFO_TYPE problem maybe just a
-   * bad GTK build on my laptop.
-   */
-  /* BASE_INFO_TYPE); */
+                              BASE_INFO_TYPE);
 
   model = GTK_TREE_MODEL (store);
 
@@ -114,7 +106,6 @@ set_info (GtkTreeModel *model,
    */
   if (root != NULL)
     {
-      base_info_ref (info); /* FIXME once boxed types are working */
       gtk_tree_store_set (store, root,
                           MODEL_COLUMN_INFO, info,
                           -1);
@@ -134,6 +125,8 @@ set_info (GtkTreeModel *model,
                          interface_info_get_methods ((InterfaceInfo*)info));
       append_child_list (model, root,
                          interface_info_get_signals ((InterfaceInfo*)info));
+      append_child_list (model, root,
+                         interface_info_get_properties ((InterfaceInfo*)info));
       break;
     case INFO_TYPE_METHOD:
       append_child_list (model, root,
@@ -143,6 +136,9 @@ set_info (GtkTreeModel *model,
       append_child_list (model, root,
                          signal_info_get_args ((SignalInfo*)info));
       break;
+    case INFO_TYPE_PROPERTY:
+      /* no children */
+      break;
     case INFO_TYPE_ARG:
       /* no children */
       break;
@@ -292,8 +288,17 @@ info_set_func_text (GtkTreeViewColumn *tree_column,
     case INFO_TYPE_SIGNAL:
       g_string_append (str, "<i>signal</i>");
       break;
+    case INFO_TYPE_PROPERTY:
+      g_string_append (str, "<i>property</i>");
+      g_string_append_printf (str, " <b>%s</b>",
+                              property_info_get_type ((PropertyInfo*)info));
+      break;
     case INFO_TYPE_ARG:
-      g_string_append (str, "<i>arg</i>");
+      g_string_append_printf (str, "<i>arg</i> %s",
+                              arg_info_get_direction ((ArgInfo*)info) == ARG_IN ?
+                              "in" : "out");
+      g_string_append_printf (str, " <b>%s</b>",
+                              arg_info_get_type ((ArgInfo*)info));
       break;
     }
 
